@@ -1,7 +1,7 @@
 
 ##  digest -- hash digest functions for R
 ##
-##  Copyright (C) 2003 - 2019  Dirk Eddelbuettel <edd@debian.org>
+##  Copyright (C) 2003 - 2023  Dirk Eddelbuettel <edd@debian.org>
 ##  Copyright (C) 2009 - 2019  Henrik Bengtsson
 ##  Copyright (C) 2012 - 2019  Hannes Muehleisen
 ##  Copyright (C) 2014 - 2019  Jim Hester
@@ -24,7 +24,8 @@
 ##  along with digest.  If not, see <http://www.gnu.org/licenses/>.
 
 digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
-                                  "xxhash32", "xxhash64", "murmur32", "spookyhash"),
+                                  "xxhash32", "xxhash64", "murmur32",
+                                  "spookyhash", "blake3", "crc32c"),
                    serialize=TRUE,
                    file=FALSE,
                    length=Inf,
@@ -35,8 +36,12 @@ digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
                    errormode=c("stop","warn","silent"),
                    serializeVersion=.getSerializeVersion()) {
 
-    algo <- match.arg(algo)
-    errormode <- match.arg(errormode)
+    # Explicitly specify choices; this is much faster than having match.arg()
+    # infer them from the function's formals.
+    algo <- match.arg(algo, c("md5", "sha1", "crc32", "sha256", "sha512",
+                              "xxhash32", "xxhash64", "murmur32",
+                              "spookyhash", "blake3", "crc32c"))
+    errormode <- match.arg(errormode, c("stop", "warn", "silent"))
 
     if (is.infinite(length)) {
         length <- -1               # internally we use -1 for infinite len
@@ -47,32 +52,31 @@ digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
         file <- TRUE                  	# nocov
     }
 
-    streaming_algos <- c("spookyhash")
-    non_streaming_algos <- c("md5", "sha1", "crc32", "sha256", "sha512",
-                             "xxhash32", "xxhash64", "murmur32")
-    if(algo %in% streaming_algos && !serialize){
+    is_streaming_algo <- algo == "spookyhash"
+
+    if (is_streaming_algo && !serialize) {
         .errorhandler(paste0(algo, " algorithm is not available without serialization."),  # #nocov
                       mode=errormode)                                                      # #nocov
     }
 
     if (serialize && !file) {
-        if(algo %in% non_streaming_algos){
-            ## support the 'nosharing' option in pqR's base::serialize()
+        if (!is_streaming_algo) {
+            ## support the 'nosharing' option in pqR's serialize()
             object <- if (.hasNoSharing())
-                base::serialize (object, connection=NULL, ascii=ascii,
+                serialize (object, connection=NULL, ascii=ascii,
                                  nosharing=TRUE, version=serializeVersion)
             else
-                base::serialize (object, connection=NULL, ascii=ascii,
+                serialize (object, connection=NULL, ascii=ascii,
                                  version=serializeVersion)
         }
         ## we support raw vectors, so no mangling of 'object' is necessary
         ## regardless of R version
         ## skip="auto" - skips the serialization header [SU]
-        if (any(!is.na(pmatch(skip,"auto"))))
+        if (is.character(skip) && skip == "auto")
             skip <- set_skip(object, ascii)
 
     } else if (!is.character(object) && !inherits(object,"raw") &&
-               algo %in% non_streaming_algos) {
+               !is_streaming_algo) {
         return(.errorhandler(paste("Argument object must be of type character",		    # #nocov
                                    "or raw vector if serialize is FALSE"), mode=errormode)) # #nocov
     }
@@ -80,7 +84,7 @@ digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
         return(.errorhandler("file=TRUE can only be used with a character object",          # #nocov
                              mode=errormode))                                               # #nocov
 
-    if (file && algo %in% streaming_algos)
+    if (file && is_streaming_algo)
         return(.errorhandler(paste0(algo, " algorithm can not be used with files."),        # #nocov
                              mode=errormode))                                               # #nocov
 
@@ -98,7 +102,7 @@ digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
     ## into 0 because auto should have been converted into a number earlier
     ## if it was valid [SU]
     if (is.character(skip)) skip <- 0
-    if (algo %in% non_streaming_algos) {
+    if (!is_streaming_algo) {
         val <- .Call(digest_impl,
                      object,
                      as.integer(algoint),
@@ -108,7 +112,7 @@ digest <- function(object, algo=c("md5", "sha1", "crc32", "sha256", "sha512",
                      as.integer(seed))
     } else if (algo == "spookyhash"){
         # 0s are the seeds. They are included to enable testing against fastdigest.
-        val <- paste(.Call(spookydigest_impl, object, skip, 0, 0, serializeVersion), collapse="")
+        val <- paste(.Call(spookydigest_impl, object, skip, 0, 0, serializeVersion, NULL), collapse="")
     }
 
     ## crc32 output was not guaranteed to be eight chars long, which we corrected
@@ -144,7 +148,9 @@ algo_int <- function(algo)
         xxhash32 = 6,
         xxhash64 = 7,
         murmur32 = 8,
-        spookyhash = 9
+        spookyhash = 9,
+        blake3 = 10,
+        crc32c = 11
     )
 
 ## HB 14 Mar 2007:
